// ====================================================================
// Copyright (C) 2011 - DIGITEO - Michael Baudin
// 
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// ====================================================================

function [data,hLfil,hDrop,hPermtol,hBloc] = spilu_ilutpplot(varargin)
    // Plots the sensitivity of ILUT for A.
    // 
    // Calling Sequence
    //   spilu_ilutpplot(A)
    //   data = spilu_ilutpplot(A)
    //   data = spilu_ilutpplot(A,N)
    //   [data,hLfil] = spilu_ilutpplot(...)
    //   [data,hLfil,hDrop] = spilu_ilutpplot(...)
    //   [data,hAlpha,hDrop,hPermtol] = spilu_ilutpplot(...)
    //   [data,hAlpha,hDrop,hPermtol,hBloc] = spilu_ilutpplot(...)
    //
    // Parameters
    // A: a n-by-n sparse matrix
    // N: a 1-by-1 matrix of doubles, integer value, the number of points in the plot (default N = 100).
    // data: a N-by-6 matrix of doubles, the values of the parameters
    // hLfil: a graphics handle, the lfil plot
    // hDrop: a graphics handle, the drop plot
    // hPermtol: a graphics handle, the permtol plot
    // hBloc: a graphics handle, the bloc plot
    //
    // Description
    // For one particular matrix, plot norm(A-L*U)/norm(A) and nnz(L)+nnz(U), depending on 
    // lfil and drop for the ILUT method.
    //
    // The columns of the array data are:
    // <itemizedlist>
    //     <listitem><para>
    //         data(:,1) : the values of lfil
    //     </para></listitem>
    //     <listitem><para>
    //         data(:,2) : the values of norm(A-L*U)/norm(A) with respect to lfil
    //     </para></listitem>
    //     <listitem><para>
    //         data(:,3) : the values of nnz(L)+nnz(U) with respect to lfil
    //     </para></listitem>
    //     <listitem><para>
    //         data(:,4) : the values of drop
    //     </para></listitem>
    //     <listitem><para>
    //         data(:,5) : the values of norm(A-L*U)/norm(A) with respect to drop
    //     </para></listitem>
    //     <listitem><para>
    //         data(:,6) : the values of nnz(L)+nnz(U) with respect to drop
    //     </para></listitem>
    //     <listitem><para>
    //         data(:,7) : the values of permtol
    //     </para></listitem>
    //     <listitem><para>
    //         data(:,8) : the values of norm(P*A-L*U)/norm(A) with respect to permtol
    //     </para></listitem>
    //     <listitem><para>
    //         data(:,9) : the values of nnz(L)+nnz(U) with respect to permtol
    //     </para></listitem>
    //     <listitem><para>
    //         data(:,10) : the values of bloc
    //     </para></listitem>
    //     <listitem><para>
    //         data(:,11) : the values of norm(P*A-L*U)/norm(A) with respect to bloc
    //     </para></listitem>
    //     <listitem><para>
    //         data(:,12) : the values of nnz(L)+nnz(U) with respect to bloc
    //     </para></listitem>
    // </itemizedlist>
    // 
    // If the decomposition does not work for one value of the parameter, 
    // then norm(A-L*U)/norm(A) and nnz(L)+nnz(U) are set to Infinity.
    // 
    // Examples
    // A = [
    //   10.     -6.    -1.     0.     3.  
    //   -6.     34.     0.    10.     0.  
    //   -1.      0.     1.     0.     0.  
    //    0.     10.     0.    10.     0.  
    //    3.      0.     0.     0.     3.  
    // ];
    // A = sparse(A);
    // spilu_ilutpplot(A);
    //
    // // See on a 225-by-225 sparse matrix
    // path = spilu_getpath (  );
    // filename = fullfile(path,"tests","matrices","pde225.mtx");
    // A=mmread(filename);
    // [data,hLfil,hDrop,hPermtol,hBloc] = spilu_ilutpplot(A);
    //
    // Authors
    //   Copyright (C) 2011 - DIGITEO - Michael Baudin

    function [pNorm,pNnz] = ilutpCompute(L,U,perm,ierr)
        P = spilu_permVecToMat(perm);
        pNorm = norm(P*A-L*U,"inf")/norm(A,"inf");            
        pNnz = nnz(L) + nnz(U)
    endfunction

    function [pNorm,pNnz] = ilutpDecompLfil(A,lfil)
        instr = "[L,U,perm]=spilu_ilutp(A,lfil)";
        ierr = execstr(instr,"errcatch");
        if (ierr<>0) then
            pNorm = %inf
            pNnz = %inf
        else
            [pNorm,pNnz] = ilutpCompute(L,U,perm,ierr)
        end
    endfunction

    function [pNorm,pNnz] = ilutpDecompDrop(A,drop)
        instr = "[L,U,perm]=spilu_ilutp(A,[],drop)";
        ierr = execstr(instr,"errcatch");
        if (ierr<>0) then
            pNorm = %inf
            pNnz = %inf
        else
            [pNorm,pNnz] = ilutpCompute(L,U,perm,ierr)
        end
    endfunction

    function [pNorm,pNnz] = ilutpDecompPermtol(A,permtol)
        instr = "[L,U,perm]=spilu_ilutp(A,[],[],permtol)";
        ierr = execstr(instr,"errcatch");
        if (ierr<>0) then
            pNorm = %inf
            pNnz = %inf
        else
            [pNorm,pNnz] = ilutpCompute(L,U,perm,ierr)
        end
    endfunction

    function [pNorm,pNnz] = ilutpDecompBloc(A,bloc)
        instr = "[L,U,perm]=spilu_ilutp(A,[],[],[],bloc)";
        ierr = execstr(instr,"errcatch");
        if (ierr<>0) then
            pNorm = %inf
            pNnz = %inf
        else
            [pNorm,pNnz] = ilutpCompute(L,U,perm,ierr)
        end
    endfunction

    [lhs,rhs]=argn()
    apifun_checkrhs ( "spilu_ilutpplot" , rhs , 1:2 )
    apifun_checklhs ( "spilu_ilutpplot" , lhs , 0:5 )
    // 
    // Get arguments
    A = varargin(1)
    N = apifun_argindefault ( varargin , 2 , 100 )
    //
    // Check type
    apifun_checktype ( "spilu_ilutpplot" , A , "A" , 1 , "sparse" )
    apifun_checktype ( "spilu_ilutpplot" , N , "N" , 2 , "constant" )
    //
    // Check size
    apifun_checksquare ( "spilu_ilutpplot" , A , "A" , 1 )
    apifun_checkscalar ( "spilu_ilutpplot" , N , "N" , 2 )
    //
    // Check content
    apifun_checkgreq ( "spilu_ilutpplot" , N , "N" , 2 , 1 )
    //
    // Proceed...
    //
    // Compute decomposition for various values of lfil
    n = size(A,"r")
    lfil = floor(linspace(0,n,N)')
    hLfil = scf();
    [pNormLfil,pNnzLfil] = spilu_plotparameter(A,"lfil",lfil,list(ilutpDecompLfil),%t)
    //
    // Compute decomposition for various values of drop
    drop = max(abs(A)) * logspace(-8,0,N)'
    hDrop = scf();
    [pNormDrop,pNnzDrop] = spilu_plotparameter(A,"drop",drop,list(ilutpDecompDrop),%t)
    hDrop.children(1).log_flags = "lnn"
    hDrop.children(2).log_flags = "lnn"
    //
    // Compute decomposition for various values of permtol
    permtol = max(abs(A)) * logspace(-5,0,N)'
    hPermtol = scf();
    [pNormPermtol,pNnzPermtol] = spilu_plotparameter(A,"permtol",permtol,list(ilutpDecompPermtol),%t)
    hPermtol.children(1).log_flags = "lnn"
    hPermtol.children(2).log_flags = "lnn"
    //
    // Compute decomposition for various values of bloc
    n = size(A,"r")
    bloc = floor(linspace(1,n,N)')
    hBloc = scf();
    [pNormBloc,pNnzBloc] = spilu_plotparameter(A,"bloc",bloc,list(ilutpDecompBloc),%t)
    //
    // Gather the results
    data = [...
    lfil,pNormLfil,pNnzLfil,..
    drop,pNormDrop,pNnzDrop,...
    permtol,pNormPermtol,pNnzPermtol,...
    bloc,pNormBloc,pNnzBloc...
    ]
endfunction
